import es, cfglib, services

info = es.AddonInfo()
info.name           = "Safe RCON"
info.version        = "2.0.1"
info.author         = "DanielB"
info.url            = "http://addons.eventscripts.com/srcon"
info.description    = "Allow admins to change server variables and access server commands without giving them rcon!" 

es.ServerVar('srcon_ver', info.version, 'Safe Rcon by DanielB').makepublic()


addondir = str(es.ServerVar('eventscripts_addondir'))

##################################################
# Classes/Global Class Instances
##################################################

class Settings(object):
    config = cfglib.AddonCFG(addondir + '/srcon/settings.cfg')
    
    defCommands = 'sv_gravity;mp_restartgame;sv_alltalk;mp_footsteps;mp_flashlight;sv_airaccelerate;sv_maxspeed;mp_freezetime;exec;es_load;es_unload'
    
    _prefix = config.cvar('srcon_prefix', '/', 'What should commands be prefixed with?'), str
    _commands  = config.cvar('srcon_commands', defCommands, 'What commands to allow or block (according to srcon_blockorallow)'), str
    _boa  = config.cvar('srcon_blockorallow', 'allow', 'What commands to allow or block'), str
    _allowall  = config.cvar('srcon_allowall', '0', 'If 1, any command can be run'), int
    _blockedprefixs = config.cvar('srcon_blockedprefixs', 'ma_;sm_;xa;rcon;alias;srcon', 'Block command prefixes (such as ma_ to stop mani admin via rcon)'), str
    
    _admins = config.cvar('srcon_admins', 'STEAM_ID_LAN;', 'Steamids of people who can use this addon. Not used if an auth service is loaded.'), str
    
    config.execute()
    
    def __init__(self):
        self.config.write()
        
    def __getattr__(self, name):
        if hasattr(Settings, name):
            return super(Settings, self).__getattribute__(name)
           
        elif hasattr(Settings, '_'+name):
            val, func = super(Settings, self).__getattribute__('_'+name)
            return func(val)
           
        raise AttributeError, 'No such attribute "%s"'%name
settings = Settings()


##################################################
# load/Unload
##################################################
def load():
    setupAuth()
    es.addons.registerSayFilter(chatter)
    
def unload():
    es.addons.unregisterSayFilter(chatter)
    
##################################################
# Misc Functions
##################################################
    
def tell(userid, text):
    es.tell(userid, '#multi', '#green[SRCON]#lightgreen ' + text)

def setupAuth():
    global isAuthed
    if services.isRegistered('auth'):
        auth_service = services.use('auth')
        auth_service.registerCapability('safe_rcon', auth_service.ADMIN)
        isAuthed = lambda x: auth_service.isUseridAuthorized(x, 'safe_rcon')
    else:
        isAuthed = lambda x: es.getplayersteamid(x) in settings.admins.split(';')
        

def prefixFail(command):
    for prefix in settings.blockedprefixs.split(';'):
        if command.startswith(prefix):
            return True
    return False

def chatter(userid, text, teamonly):
    prefix = settings.prefix
    text = text.strip('"')
    
    if text.startswith(prefix):
        if isAuthed(userid):
            commandString = text[len(prefix):].split(';')[0] # Ensure they only use one command!
            command = commandString.split(' ')[0]
            if (settings.allowall) or (settings.boa == 'allow' and command in settings.commands) or (settings.boa == 'block' and not command in settings.commands):
                if prefixFail(command):
                    tell(userid, 'You cannot run that command')
                else:
                    es.server.queuecmd(commandString)
                return (None, None, None)
            else:
                tell(userid, 'That command is blocked.')
                
    return (userid, text, teamonly)
    